#!/usr/bin/env bash

# Installs Banyan Netagent as an upstart or systemd service.

DIR=$( cd "$( dirname "$0" )" && pwd )

DIRBIN=${DIR}/x/bin
DIRETC=${DIR}/x/etc
DIRKMS=${DIR}/x/kms

config_file=${DIR}/config.yaml
log_home=/var/log/banyan

# Install dependencies by default
INSTALL_DEPS=${INSTALL_DEPS:-true}

if [[ $# -ne 0 ]]; then
	echo "Usage: $0"
	exit 1
fi

if [[ ! -f ${config_file} ]]; then
  echo "Error: Missing config file ${config_file}"
	exit 1
fi

# Prepare to install dependencies
function install_deps_prepare {
  # try to locate yum, fall back to apt-get
  if [[ $(command -v yum) ]]; then
    sudo yum clean metadata
    check_error "sudo yum clean metadata"
  else
    sudo DEBIAN_FRONTEND=noninteractive apt-get -q=2 update
    check_error "sudo apt-get update"
  fi
}

# Install dependencies, exit on any error
function install_deps {
  local deps=("${@}")

  # try to locate yum, fall back to apt-get
  if [[ $(command -v yum) ]]; then
    sudo yum -y -q install "${deps[@]}"
    check_error "sudo yum install ${deps[*]}"
  else
    sudo DEBIAN_FRONTEND=noninteractive apt-get -q=2 -y install "${deps[@]}"
    check_error "sudo apt-get install ${deps[*]}"
    setup_cgroup
  fi
}

# Install dependencies, do NOT exit on error
function install_deps_without_exit {
  local deps=("${@}")

  if [[ $(command -v yum) ]]; then
    sudo yum -y -q install "${deps[@]}" 2> /dev/null
  else
    sudo DEBIAN_FRONTEND=noninteractive apt-get -q=2 -y install "${deps[@]}" 2> /dev/null
  fi
}

function check_error {
	local err=$?
	if [[ ${err} -ne 0 ]]; then
		echo "$1 Error: ${err}"
		exit 1
	fi
}

function yaml_value {
	awk "/$1/"'{n=split($0,a,":"); if (n<=2) {print a[n]} else {joined=a[2]; for (i=3; i<=n; i++) {joined = joined ":" a[i]}; print joined}}' ${config_file}| sed -e 's/"//g' | sed -e "s/'//g"
}

function setup_cgroup {
  # return if cpu cgroup subsystem mounted
	[[ -f /sys/fs/cgroup/cpu ]] && return

	# return if os-release is missing
	[[ -f /etc/os-release ]] || return

	# return if not 14.04
	grep -q 14.04 /etc/os-release || return

	# looks like ubuntu 14.04
	sudo DEBIAN_FRONTEND=noninteractive apt-get -q=2 -y install cgroup-lite
	check_error "apt-get install cgroup-lite"

	sudo cgroups-mount
}

# validate config values
shield_addr=$(yaml_value shield_address)
if [[ -z ${shield_addr} ]]; then
	echo "Error: shield_address not defined in ${config_file}"
	exit 1
fi
echo "Expecting Shield to be at ${shield_addr}"

sudo install -d -m 0755 /opt/banyan
check_error "install -d -m 0755 /opt/banyan"

sudo install -d -m 0755 ${log_home}
check_error "install -d -m 0755 ${log_home}"

echo "Logs will be stored in ${log_home}/netagent.log"

# install dependencies
if [[ ${INSTALL_DEPS} == "true" ]]; then
	install_deps_prepare
	install_deps ipset ebtables
	grep -q -v Amazon /etc/os-release && install_deps_without_exit wireguard unbound
fi

[[ -d /opt/banyan/local ]] && sudo rm -rf /opt/banyan/local

sudo tar -C /opt/banyan -xzf ${DIRBIN}/local_iptables.tar.gz
check_error "sudo tar -C /opt/banyan -xzf ${DIRBIN}/local_iptables.tar.gz"

INIT_SYSTEM_CMD=(sudo $(command -v systemctl || command -v initctl))
"${INIT_SYSTEM_CMD[@]}" stop netagent

# Install executable
sudo install -m 0700 -C -t /opt/banyan ${DIRBIN}/netagent

# Install configuration
sudo install -m 0600 -C -t /opt/banyan ${DIR}/config.yaml

# Use process id 1 to determine init system, default to upstart
if [[ $(ps -p1 | grep -o systemd) == "systemd" ]]; then
	echo "Looks like a systemd system"
	sudo install -m 0644 "${DIRETC}/netagent.service.tpl" /lib/systemd/system/netagent.service
	check_error "install -m 0644 ${DIRETC}/netagent.service.tpl /lib/systemd/system/netagent.service"
	"${INIT_SYSTEM_CMD[@]}" enable netagent
	"${INIT_SYSTEM_CMD[@]}" daemon-reload
else
	echo "Looks like an upstart system"
	sudo install -m 0644 "${DIRETC}/netagent.conf.tpl" /etc/init/netagent.conf
	check_error "install -m 0644 ${DIRETC}/netagent.conf.tpl /etc/init/netagent.conf"
fi

"${INIT_SYSTEM_CMD[@]}" stop netagent
"${INIT_SYSTEM_CMD[@]}" start netagent
"${INIT_SYSTEM_CMD[@]}" status netagent
